// src/index.ts
import { createRequire } from "module";
import { createSyncFn } from "synckit";
var ExtraPosition = /* @__PURE__ */ ((ExtraPosition2) => {
  ExtraPosition2[ExtraPosition2["before"] = 0] = "before";
  ExtraPosition2[ExtraPosition2["after"] = 1] = "after";
  return ExtraPosition2;
})(ExtraPosition || {});
function getThemeName(theme) {
  if (typeof theme === "string")
    return theme;
  return theme.name;
}
var HIGHLIGHT_RE = /{([\d,-]+)}/;
var BACKGROUND_STYLE_RE = /^<pre[^>]*style="([^"]*)"[^>]*>/;
var DARK_CLASS = "shiki-dark";
var LIGHT_CLASS = "shiki-light";
function resolveOptions(options) {
  const themes = [];
  let darkModeThemes;
  if (!options.theme) {
    themes.push("nord");
  } else if (typeof options.theme === "string") {
    themes.push(options.theme);
  } else {
    if ("dark" in options.theme || "light" in options.theme) {
      darkModeThemes = options.theme;
      themes.push(darkModeThemes.dark);
      themes.push(darkModeThemes.light);
    } else {
      themes.push(options.theme);
    }
  }
  return {
    ...options,
    themes,
    darkModeThemes: darkModeThemes ? {
      dark: getThemeName(darkModeThemes.dark),
      light: getThemeName(darkModeThemes.light)
    } : void 0,
    extra: options.extra || []
  };
}
var attrsToLines = (attrs) => {
  const result = [];
  if (!attrs.trim())
    return [];
  attrs.split(",").map((v) => v.split("-").map((v2) => parseInt(v2, 10))).forEach(([start, end]) => {
    if (start && end) {
      result.push(
        ...Array.from({ length: end - start + 1 }, (_, i) => start + i)
      );
    } else {
      result.push(start);
    }
  });
  return result.map((v) => ({
    line: v,
    classes: ["highlighted"]
  }));
};
var processExtra = (extra, attrs) => {
  const result = [];
  if (extra === void 0)
    return result;
  extra.forEach((processor) => {
    const matched = processor.attrRe ? processor.attrRe.exec(attrs) : null;
    result.push({
      light: processor.light(matched),
      dark: processor.dark === null ? void 0 : (processor.dark || processor.light)(matched),
      position: processor.position
    });
  });
  return result;
};
var getStyleContent = (tag, regex) => {
  const match = regex.exec(tag);
  let style_content = "";
  if (match)
    style_content = match[1];
  return style_content;
};
var appendStyle = (intel, style) => {
  if (!intel)
    return;
  if (!style.endsWith(";"))
    style += ";";
  if (intel.attrs.style)
    intel.attrs.style += `;${style}`;
  else
    intel.attrs.style = style;
};
var appendClass = (intel, className) => {
  if (!intel)
    return;
  if (intel.attrs.class)
    intel.attrs.class += ` ${className}`;
  else
    intel.attrs.class = className;
};
var h = (intel) => {
  if (!intel)
    return "";
  const headPartal = `<${intel.tag} ${Object.entries(intel.attrs).reduce((prev, now) => {
    return `${prev} ${now[0]}="${now[1]}"`;
  }, "")}`;
  if (intel.content)
    return `${headPartal}>${intel.content}</${intel.tag}>`;
  else
    return `${headPartal} />`;
};
var wrapFinalContainer = (light, dark = void 0, processedExtra = void 0) => {
  let prependResult = "";
  let appendResult = "";
  dark = dark || "";
  processedExtra = processedExtra || [];
  const light_style_content = getStyleContent(light, BACKGROUND_STYLE_RE);
  const dark_style_content = getStyleContent(dark, BACKGROUND_STYLE_RE);
  processedExtra.forEach((extra) => {
    appendClass(extra.light, LIGHT_CLASS);
    appendStyle(extra.light, light_style_content);
    appendClass(extra.dark, DARK_CLASS);
    appendStyle(extra.dark, dark_style_content);
    const lightExtra = h(extra.light);
    const darkExtra = h(extra.dark);
    if (extra.position === 0 /* before */) {
      prependResult += lightExtra;
      prependResult += darkExtra;
    } else {
      appendResult += lightExtra;
      appendResult += darkExtra;
    }
  });
  return `<div class="shiki-container">${prependResult}${dark}${light}${appendResult}</div>`;
};
var MarkdownItShiki = (markdownit, options = {}) => {
  const _highlighter = options.highlighter;
  const {
    langs,
    themes,
    darkModeThemes,
    highlightLines,
    extra
  } = resolveOptions(options);
  let syncRun;
  if (!_highlighter) {
    const require2 = createRequire(import.meta.url);
    syncRun = createSyncFn(require2.resolve("./worker"));
    syncRun("getHighlighter", { langs, themes });
  }
  const highlightCode = (code, lang, theme, lineOptions) => {
    if (_highlighter)
      return _highlighter.codeToHtml(code, { lang: lang || "text", theme, lineOptions });
    return syncRun("codeToHtml", {
      code,
      theme,
      lang: lang || "text",
      lineOptions
    });
  };
  markdownit.options.highlight = (code, lang, attrs) => {
    let lineOptions;
    if (highlightLines) {
      const match = HIGHLIGHT_RE.exec(attrs);
      if (match)
        lineOptions = attrsToLines(match[1]);
    }
    const processedExtra = processExtra(extra, attrs);
    if (darkModeThemes) {
      const dark = highlightCode(code, lang, darkModeThemes.dark, lineOptions).replace('<pre class="shiki', `<pre class="shiki ${DARK_CLASS}`);
      const light = highlightCode(code, lang || "text", darkModeThemes.light, lineOptions).replace('<pre class="shiki', `<pre class="shiki ${LIGHT_CLASS}`);
      return wrapFinalContainer(light, dark, processedExtra);
    } else {
      return wrapFinalContainer(
        highlightCode(
          code,
          lang || "text",
          void 0,
          lineOptions
        ),
        void 0,
        processedExtra
      );
    }
  };
};
var src_default = MarkdownItShiki;

export {
  ExtraPosition,
  resolveOptions,
  h,
  src_default
};
